@@ -19,7 +19,8 @@ class Mongodb:
     ORG_CONFIG_COLLECTION = 'org_config'
     COUNTERS_COLLECTION = 'counters'
     EVENTS_COLLECTION = 'events'
-    DAILY_REPORTS_COLLECTION = 'daily_reports'
+    CANCELLATION_POLICIES = 'cancellationPolicies'
+    DAILY_REPORTS_COLLECTION = 'daily_report'
 
     def __init__(self, db_name):
         try:
@@ -67,4 +68,4 @@ if __name__ == '__main__':
     mongo = Mongodb('11249')
     val = mongo.get_collection(mongo.GUEST_COLLECTION).count()
     print(val)
-    mongo.close_connection()
\ No newline at end of file
+    mongo.close_connection()
