@@ -1,7 +1,7 @@
 import json
 from datetime import datetime
 
-from upswingutil.db.model import Status, ReservationGuestInfo, ReservationGuestShort, GuestsModel, \
+from upswingutil.db.model import Status, ReservationGuestInfo, ReservationGuestShort, \
     ReservationModelV2, GuestsModelV2
 from upswingutil.resource import get_model_from_cloud_storage
 from upswingutil.pms.rms import NAME, get_key, validate_key
@@ -9,8 +9,10 @@ from upswingutil.db import Mongodb, Firestore, MongodbV2
 from upswingutil.schema import Token
 import requests
 import logging
-import asyncio
+from pycountry_convert import country_name_to_country_alpha2
 from concurrent.futures import ThreadPoolExecutor, as_completed
+from upswingutil.pms.rms import COUNTRIES
+
 
 def _get_status(status):
     if status == 'NoShow':
@@ -102,11 +104,13 @@ def _get_geo_region(nationalID: int):
 def _generate_day_to_day_entry(record):
     _daily_activity = list()
     for index, item in enumerate(record.get('daily_rates')):
+        _daily_revenue = next((x for x in record.get('daily_revenue')), {})
         _daily_activity.append({
             'date': item['stayDate'].split(' ')[0],
             'day_of_stay': index + 1,
             'roomType': record.get('categoryName'),
             'ratePlanCode': record.get('rateTypeName'),
+            'rateTypeId': item.get('rateTypeId'),
             'roomId': record.get('areaId'),
             'roomName': record.get('areaName'),
             'rates': {
@@ -118,20 +122,19 @@ def _generate_day_to_day_entry(record):
                 'dynamicAmount': item.get('dynamicAmount'),
                 'linkedRateAdjustmentAmount': item.get('linkedRateAdjustmentAmount'),
                 'packageAmount': item.get('packageAmount'),
-                'rateTypeId': item.get('rateTypeId'),
                 'xNightsDiscount': item.get('xNightsDiscount'),
                 'totalRateAmount': item.get('totalRateAmount')
             },
             'revenue': {
-                'accommodation': item.get('accommodation'),
-                'accommodationTax': item.get('accommodationTax'),
-                'accommodationGST': item.get('accommodationGST'),
-                'foodAndBeverage': item.get('foodAndBeverage'),
-                'foodAndBeverageTax': item.get('foodAndBeverageTax'),
-                'foodAndBeverageGST': item.get('foodAndBeverageGST'),
-                'other': item.get('other'),
-                'otherTax': item.get('otherTax'),
-                'otherGST': item.get('otherGST')
+                'accommodation': _daily_revenue.get('accommodation'),
+                'accommodationTax': _daily_revenue.get('accommodationTax'),
+                'accommodationGST': _daily_revenue.get('accommodationGST'),
+                'foodAndBeverage': _daily_revenue.get('foodAndBeverage'),
+                'foodAndBeverageTax': _daily_revenue.get('foodAndBeverageTax'),
+                'foodAndBeverageGST': _daily_revenue.get('foodAndBeverageGST'),
+                'other': _daily_revenue.get('other'),
+                'otherTax': _daily_revenue.get('otherTax'),
+                'otherGST': _daily_revenue.get('otherGST')
             },
             'guestCounts': {
                 'adults': record.get('adults'),
@@ -142,60 +145,6 @@ def _generate_day_to_day_entry(record):
     return _daily_activity
 
 
-def _transform_guest_profile(record):
-    guest = GuestsModelV2(
-        _id=str(record.get('id')),
-        idObj=_create_guest_id_obj(record),
-        profileType='Guest',
-        title=record.get('title'),
-        firstName=record.get('guestGiven'),
-        lastName=record.get('guestSurname'),
-        datesAndDurations={
-            'birthDate': record.get('birthDate'),
-            'anniversary': None
-        },
-        address={
-            'addressLine1': record.get('addressLine1'),
-            'addressLine2': record.get('addressLine2'),
-            'addressLine3': record.get('addressLine3'),
-            'countryId': record.get('countryId'),
-            'nationalityId': record.get('nationalityId'),
-            'state': record.get('state'),
-            'town': record.get('town'),
-            'postCode': record.get('postCode'),
-        },
-        contactInfo={
-            'email': record.get('email'),
-            'email2': record.get('email2'),
-            'mobile': record.get('mobile'),
-            'telephone': record.get('phoneAH'),
-        },
-        privacyInfo={
-            'emailOptOut': record.get('emailOptOut'),
-            'marketingOptOut': record.get('marketingOptOut'),
-            'smsOptOut': record.get('smsOptOut'),
-            'privacyOptIn': record.get('privacyOptIn'),
-            'phoneOptOut': record.get('phoneOptOut'),
-        },
-        documents={
-            'passportId': record.get('passportId'),
-        },
-        metaInfo={
-            'notes': record.get('notes'),
-            'languageSpokenId': record.get('languageSpokenId'),
-            'contracts': record.get('contracts'),
-            'accounts': record.get('accounts'),
-            'mealCard': record.get('mealCard')
-        },
-        company={
-          'companyId': record.get('companyId')
-        },
-        registeredProperty=str(record.get('propertyId')),
-        auraRecordUpdatedOn=datetime.now(),
-    )
-    return guest
-
-
 def _add_reservation_to_alvie(record):
     firestore_db = Firestore(app='alvie')
     user_email_list = [item["email"] for item in record.get('guests')]
@@ -294,6 +243,69 @@ class ReservationSync:
         finally:
             return result
 
+    def _transform_guest_profile(self, record):
+        countryCode = country_name_to_country_alpha2(
+            next((x['name'] for x in COUNTRIES if x['id'] == record.get('countryId')), ''))
+        if countryCode is None:
+            logging.error(f'No countryCode found for {record.get("countryId")}')
+        nationalityCode = country_name_to_country_alpha2(
+            next((x['name'] for x in COUNTRIES if x['id'] == record.get('nationalityId')), ''))
+        if nationalityCode is None:
+            logging.error(f'No countryCode found for {record.get("nationalityId")}')
+        hotelId = self.mongo.get_collection(self.mongo.PROPERTY_COLLECTION).find_one({'primaryDetails.propertyId': str(record.get('propertyId'))}, {'_id': 1})
+        hotelId = hotelId if hotelId else '0'
+        guest = GuestsModelV2(
+            _id=str(record.get('id')),
+            idObj=_create_guest_id_obj(record),
+            profileType='Guest',
+            title=record.get('title'),
+            firstName=record.get('guestGiven'),
+            lastName=record.get('guestSurname'),
+            datesAndDurations={
+                'birthDate': record.get('birthDate'),
+                'anniversary': None
+            },
+            address={
+                'addressLine1': record.get('addressLine1'),
+                'addressLine2': record.get('addressLine2'),
+                'addressLine3': record.get('addressLine3'),
+                'countryId': countryCode,
+                'nationalityId': nationalityCode,
+                'state': record.get('state'),
+                'town': record.get('town'),
+                'postCode': record.get('postCode'),
+            },
+            contactInfo={
+                'email': record.get('email'),
+                'email2': record.get('email2'),
+                'mobile': record.get('mobile'),
+                'telephone': record.get('phoneAH'),
+            },
+            privacyInfo={
+                'emailOptOut': record.get('emailOptOut'),
+                'marketingOptOut': record.get('marketingOptOut'),
+                'smsOptOut': record.get('smsOptOut'),
+                'privacyOptIn': record.get('privacyOptIn'),
+                'phoneOptOut': record.get('phoneOptOut'),
+            },
+            documents={
+                'passportId': record.get('passportId'),
+            },
+            metaInfo={
+                'notes': record.get('notes'),
+                'languageSpokenId': record.get('languageSpokenId'),
+                'contracts': record.get('contracts'),
+                'accounts': record.get('accounts'),
+                'mealCard': record.get('mealCard')
+            },
+            company={
+                'companyId': record.get('companyId')
+            },
+            registeredProperty=hotelId,
+            auraRecordUpdatedOn=datetime.now(),
+        )
+        return guest
+
     def _retrieve_data(self, id, name, url):
         result = None
         try:
@@ -400,15 +412,19 @@ class ReservationSync:
                     threads.append(executor.submit(self._retrieve_data, record.get("id"), item.get("name"), item.get("url")))
 
                 for th in as_completed(threads):
-                    key, value = th.result()
-                    if key == 'subMarketSegment':
-                        value = next((x for x in value if record['subMarketSegmentId'] == x['id']), None)
-                    record[key] = value
-
-            return record
+                    try:
+                        key, value = th.result()
+                        if key == 'subMarketSegment':
+                            value = next((x for x in value if record['subMarketSegmentId'] == x['id']), None)
+                        record[key] = value
+                    except Exception as e:
+                        logging.error(f'Error retrieving addition reservation details for {record.get("id")}')
+                        logging.error(e)
         except Exception as e:
             logging.error(f'Error retrieving addition reservation details for {record.get("id")}')
             logging.error(e)
+        finally:
+            return record
 
     def _extract_guest_profile(self, guest):
         guestId = 0
@@ -450,10 +466,7 @@ class ReservationSync:
         _guest_list = list()
         try:
             for guest in record.get('guests'):
-                # loop = asyncio.get_event_loop()
-                # future = asyncio.ensure_future(self._extract_guest_profile(guest))
-                # loop.run_until_complete(future)
-                _guest_profile = _transform_guest_profile(self._extract_guest_profile(guest))
+                _guest_profile = self._transform_guest_profile(self._extract_guest_profile(guest))
                 self.mongoV2.save(_guest_profile)
                 _guest_list.append(ReservationGuestShort(
                     guest=str(guest.get('id')),
@@ -504,6 +517,10 @@ class ReservationSync:
         finally:
             return result
 
+    def _get_cancellation_policy(self, policyId):
+        result = self.mongo.get_collection(self.mongo.CANCELLATION_POLICIES).find_one({'_id': policyId})
+        return result
+
     def transform_reservation(self, record):
         try:
             logging.debug(f'Transforming reservation {record.get("id")}')
@@ -565,6 +582,7 @@ class ReservationSync:
                     'roomId': record.get('areaId'),
                     'roomName': record.get('areaName'),
                     'categoryName': record.get('categoryName'),
+                    'numberOfRooms': 1,
                     'bedConfiguration': record.get('bedConfiguration')
                 },
                 stayInfo={
@@ -679,22 +697,22 @@ class ReservationSync:
                         }
                     ],
                     'commission': {
-                        'travelAgentCommissionPercentage': record.get('financial_info_actual').get('financial_info_actual')
+                        'travelAgentCommissionPercentage': record.get('financial_info_actual').get('travelAgentCommissionPercentage')
                     },
-                    'billTo': record.get('billTo'),
-                    'billing': record.get('billing')
+                    # 'billTo': record.get('billTo'),
+                    # 'billing': record.get('billing')
                 },
                 daily_activity=_generate_day_to_day_entry(record),
                 linkedReservation=record.get('linkedReservation'),
                 guestInfo=self._get_guest_info(record),
-                eCertificates=[record.get('existingToken')],
+                eCertificates=[record.get('existingToken') if record.get('existingToken') else {}],
                 historyEvents=record.get('auditTrail'),
                 cancellation=[
                     {
                         'reason': None,
                         'cxlDate': record.get('cancelledDate'),
                         'userId': record.get('cancelledById'),
-                        'policy': record.get('cancellationPolicyId'),
+                        'policy': self._get_cancellation_policy(record.get('cancellationPolicyId')),
                         'externalCancelId': record.get('externalCancelId'),
                         'businessLostId': record.get('businessLostId')
                     }
@@ -732,12 +750,12 @@ class ReservationSync:
                 response_json['orgId'] = self.orgId
                 response_json['agent'] = record.get('agent')
                 property_info = self.mongo.get_collection(self.mongo.PROPERTY_COLLECTION).find_one(
-                    {"areas.id": response_json.get('areaId')},
-                    {'id': 1, 'name': 1,
-                     'clientId': 1}
+                    {"rooms.roomId": str(response_json.get('areaId'))},
+                    {'primaryDetails.propertyId': 1, 'name': 1,
+                     'hotelId': 1}
                 )
-                response_json['clientId'] = property_info.get('clientId') if property_info else 0
-                response_json['propertyId'] = property_info.get('_id') if property_info else 0
+                response_json['clientId'] = property_info.get('hotelId') if property_info else 0
+                response_json['propertyId'] = property_info.get('primaryDetails.propertyId') if property_info else 0
                 response_json['propertyName'] = property_info.get('name') if property_info else 'Unknown'
                 return response_json
             except Exception as err:
@@ -761,7 +779,10 @@ class ReservationSync:
         record = self.extract_reservation(record)
         if record:
             record = self.transform_reservation(self.extract_reservation_details(record))
-            self.mongoV2.save(record)
+            if record:
+                self.mongoV2.save(record)
+            else:
+                logging.error(f'Error saving the record {record} as it is None')
             # self._add_reservation_to_alvie(record)
 
     def __del__(self):
